@@ -106,7 +106,7 @@ class BaseWritter(Writer):
         self._sstream = io.StringIO()
         super().__init__(self._sstream, width)
 
-        self._build_dir = Path(build_dir)
+        self._build_dir = Path(build_dir).resolve()
         self._suffix_to_cl = {}
         self._suffix_to_rule = {}
         self.compiler_build_opts = compiler_build_opts
@@ -136,12 +136,12 @@ class BaseWritter(Writer):
         global_build_opts = self.compiler_build_opts.get(
             compiler, BuildOptions())
         opts = opts.merge(global_build_opts)
-        includes = " ".join(["-I" + str(i) for i in opts.includes])
+        includes = " ".join(["-I \"{}\"".format(str(i)) for i in opts.includes])
         cflags = opts.cflags
         post_cflags = opts.post_cflags
         cflags = " ".join(cflags)
         post_cflags = " ".join(post_cflags)
-        rule_name = name + "_cxx"
+        rule_name = name + "_cxx_{}".format(compiler_var)
         self.rule(
             rule_name,
             "${} -MMD -MT $out -MF $out.d {} {} -c $in -o $out {}".format(compiler_var, includes, cflags, post_cflags),
@@ -171,8 +171,8 @@ class BaseWritter(Writer):
                     raise NotImplementedError("unsupported lib prefix. supported: static and path")
             lib_flags.append(lib_flag)
         libs_str = " ".join(lib_flags)
-        libpaths_str = " ".join(["-L" + str(l) for l in opts.libpaths])
-        rule_name = name + "_ld"
+        libpaths_str = " ".join(["-L \"{}\"".format(str(l)) for l in opts.libpaths])
+        rule_name = name + "_ld_{}".format(linker_name)
         self.rule(
             rule_name,
             "${} $in {} {} {} -o $out".format(linker_name, libs_str, libpaths_str, ldflags),
@@ -191,10 +191,11 @@ class BaseWritter(Writer):
         post_cflags = opts.post_cflags
         cflags = " ".join(cflags)
         post_cflags = " ".join(post_cflags)
-        rule_name = name + "_cxx"
+        rule_name = name + "_cxx_{}".format(compiler_var)
         self.rule(
             rule_name,
-            "${} {} {} -c $in /Fo$out ${}".format(compiler_var, includes, cflags, post_cflags)
+            "${} {} {} /showIncludes -c $in /Fo$out ${}".format(compiler_var, includes, cflags, post_cflags),
+            deps="msvc"
         )
         self.newline()
         return rule_name
@@ -207,7 +208,7 @@ class BaseWritter(Writer):
         libs_str = " ".join([str(l) + ".lib" for l in opts.libs])
         libpaths_str = " ".join(
             ["/LIBPATH:\"{}\"".format(str(l)) for l in opts.libpaths])
-        rule_name = name + "_ld"
+        rule_name = name + "_ld_{}".format(linker_name)
         self.rule(
             rule_name,
             "${} /link /nologo $in {} {} {} /out:$out".format(linker_name, libs_str, libpaths_str, ldflags),
@@ -220,16 +221,19 @@ class BaseWritter(Writer):
         global_build_opts = self.compiler_build_opts.get(
             compiler, BuildOptions())
         opts = opts.merge(global_build_opts)
-        includes = " ".join(["-I" + str(i) for i in opts.includes])
+        includes = " ".join(["-I \"{}\"".format(str(i)) for i in opts.includes])
         cflags = opts.cflags
         post_cflags = opts.post_cflags
         cflags = " ".join(cflags)
         post_cflags = " ".join(post_cflags)
-        rule_name = name + "_cuda"
+        rule_name = name + "_cuda_{}".format(compiler_var)
+        MMD = "-MD" if compat.InWindows else "-MMD"
         self.rule(
             rule_name,
-            "${} {} {} -c $in -o $out {}".format(compiler_var, includes, cflags, post_cflags),
-            description="nvcc cxx $out")
+            "${} {} -MT $out -MF $out.d {} {} -c $in -o $out {}".format(compiler_var, MMD, includes, cflags, post_cflags),
+            description="nvcc cxx $out",
+            depfile="$out.d",
+            deps="gcc")
         self.newline()
         return rule_name
 
@@ -238,9 +242,10 @@ class BaseWritter(Writer):
             linker, LinkOptions())
         opts = opts.merge(global_build_opts)
         ldflags = " ".join(opts.ldflags)
-        libs_str = " ".join(["-l" + str(l) for l in opts.libs])
-        libpaths_str = " ".join(["-L" + str(l) for l in opts.libpaths])
-        rule_name = name + "_ld"
+        libs_str = " ".join(["-l \"{}\"".format(str(l)) for l in opts.libs])
+        libpaths_str = " ".join(["-L \"{}\"".format(str(l)) for l in opts.libpaths])
+        rule_name = name + "_ld_{}".format(linker_name)
+        libpaths_str = " ".join(["-L \"{}\"".format(str(l)) for l in opts.libpaths])
         self.rule(
             rule_name,
             "${} $in {} {} {} -o $out".format(linker_name, libs_str, libpaths_str, ldflags),
@@ -336,8 +341,11 @@ class BaseWritter(Writer):
             target_path = self._build_dir / target_filename
         obj_files = []
         for p in source_paths:
+            assert p.exists()
             suffix = ".o"
-            obj = str(self._build_dir / (p.name + suffix))
+            obj = (self._build_dir / (p.name + suffix)).resolve()
+            assert obj.parent.exists()
+            obj = str(obj)
             obj_files.append(obj)
             rule = path_to_rule[p]
             self.build(obj, rule, str(p))
@@ -376,9 +384,14 @@ def _default_suffix_to_compiler():
 COMMON_NVCC_FLAGS = [
     '-D__CUDA_NO_HALF_OPERATORS__', '-D__CUDA_NO_HALF_CONVERSIONS__',
     '-D__CUDA_NO_HALF2_OPERATORS__', '--expt-relaxed-constexpr',
-    '-Xcompiler=\"-fPIC\"'
+    '-Xcompiler=\"-fPIC\"', '-Xcompiler=\'-O3\''
 ]
 
+COMMON_NVCC_FLAGS_WINDOWS = [
+    '--expt-relaxed-constexpr', '-Xcompiler=\'/O2\''
+]
+
+
 COMMON_MSVC_FLAGS = [
     '/MD', '/wd4819', '/wd4251', '/wd4244', '/wd4267', '/wd4275', '/wd4018',
     '/wd4190', '/EHsc'
@@ -394,7 +407,11 @@ COMMON_HIPCC_FLAGS = [
 
 
 def _default_build_options():
-    nvcc_flags = COMMON_NVCC_FLAGS.copy()
+    if compat.InWindows:
+        nvcc_flags = COMMON_NVCC_FLAGS_WINDOWS.copy()
+    else:
+        nvcc_flags = COMMON_NVCC_FLAGS.copy()
+
     if compat.InWindows:
         nvcc_flags.extend("-Xcompiler=\"{}\"".format(c)
                           for c in COMMON_MSVC_FLAGS)
@@ -538,7 +555,7 @@ def build_simple_ninja(target,
         target, build_dir, sources, includes, libs, libpaths, compile_opts,
         link_opts, target_filename, additional_cflags, additional_lflags,
         suffix_to_compiler, shared)
-    build_dir = Path(build_dir)
+    build_dir = Path(build_dir).resolve()
     with (build_dir / "build.ninja").open("w") as f:
         f.write(ninja_content)
     # TODO: check_call don't raise, this is a problem
