@@ -331,6 +331,8 @@ class CppSourceIterator(object):
         # return: (func_id, param_pair, body_pair)
         state = self.state()
         res = []  # type: List[FunctionDef]
+        func_attr_kw = set(["const", "override", "final"])
+
         for meta in self.find_identifier_prefix(prefix, full_match=full_match):
             self.reset_bracket_count().move(meta.end)
             if find_after:
@@ -367,7 +369,10 @@ class CppSourceIterator(object):
                     namespaces.append(ns)
             namespaces.append(meta.name)
             func_id = "::".join(namespaces)
-
+            iden = self.next_identifier()
+            if iden is not None:
+                if iden not in func_attr_kw:
+                    continue 
             curly_pair = self.next_curly()
             if not curly_pair:
                 if decl_only:
